@@ -17,13 +17,21 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 #
-import gconf
 import os
+import logging
+
+import gtk
+import gconf
 
-import gtk.gdk
 import libvirt
 import virtinst
-import logging
+
+_spice_error = None
+try:
+    import SpiceClientGtk
+    ignore = SpiceClientGtk
+except Exception, _spice_error:
+    logging.debug("Error importing spice: %s" % _spice_error)
 
 from virtManager.keyring import vmmKeyring
 from virtManager.secret import vmmSecret
@@ -46,7 +54,9 @@ DEFAULT_XEN_SAVE_DIR = "/var/lib/xen/dump"
 DEFAULT_VIRT_IMAGE_DIR = "/var/lib/libvirt/images"
 DEFAULT_VIRT_SAVE_DIR = "/var/lib/libvirt"
 
-class vmmConfig:
+running_config = None
+
+class vmmConfig(object):
 
     # GConf directory names for saving last used paths
     CONFIG_DIR_IMAGE = "image"
@@ -85,17 +95,26 @@ class vmmConfig:
         self.appversion = appversion
         self.conf_dir = gconf_dir
         self.conf = gconf.client_get_default()
-        self.conf.add_dir (gconf_dir,
-                           gconf.CLIENT_PRELOAD_NONE)
+        self.conf.add_dir(gconf_dir,
+                          gconf.CLIENT_PRELOAD_NONE)
 
         self.glade_dir = glade_dir
         self.icon_dir = icon_dir
         self.data_dir = data_dir
+
         # We don't create it straight away, since we don't want
         # to block the app pending user authorizaation to access
         # the keyring
         self.keyring = None
 
+        self.default_qemu_user = "root"
+
+        # Use this key to disable certain features not supported on RHEL
+        self.enable_unsupported_rhel_opts = True
+        self.preferred_distros = []
+        self.hv_packages = []
+        self.libvirt_packages = []
+
         self.status_icons = {
             libvirt.VIR_DOMAIN_BLOCKED: gtk.gdk.pixbuf_new_from_file_at_size(self.get_icon_dir() + "/state_running.png", 18, 18),
             libvirt.VIR_DOMAIN_CRASHED: gtk.gdk.pixbuf_new_from_file_at_size(self.get_icon_dir() + "/state_crashed.png", 18, 18),
@@ -148,6 +167,15 @@ class vmmConfig:
     def get_data_dir(self):
         return self.data_dir
 
+    def get_spice_error(self):
+        return _spice_error and str(_spice_error) or None
+
+    def embeddable_graphics(self):
+        ret = ["vnc"]
+        if not bool(self.get_spice_error()):
+            ret.append("spice")
+        return ret
+
     # Per-VM/Connection/Connection Host Option dealings
     def _perconn_helper(self, uri, pref_func, func_type, value=None):
         suffix = "connection_prefs/%s" % gconf.escape_key(uri, len(uri))
@@ -290,7 +318,7 @@ class vmmConfig:
 
     # Check whether we have GTK-VNC that supports configurable grab keys
     # installed on the system
-    def grab_keys_supported(self):
+    def vnc_grab_keys_supported(self):
         try:
             import gtkvnc
             return hasattr(gtkvnc.Display, "set_grab_keys")
@@ -298,7 +326,7 @@ class vmmConfig:
             return False
 
     # Keys preferences
-    def get_keys_combination(self, syms = False):
+    def get_keys_combination(self, syms=False):
         val = self.conf.get_string(self.conf_dir + "/keys/grab-keys")
         if syms == True:
             return val
@@ -425,15 +453,16 @@ class vmmConfig:
     def set_console_popup(self, pref):
         self.conf.set_int(self.conf_dir + "/console/popup", pref)
 
-    def on_console_keygrab_changed(self, callback):
-        self.conf.notify_add(self.conf_dir + "/console/keygrab", callback)
-    def get_console_keygrab(self):
-        console_pref = self.conf.get_int(self.conf_dir + "/console/keygrab")
+    def on_console_accels_changed(self, callback):
+        self.conf.notify_add(self.conf_dir + "/console/enable-accels", callback)
+    def get_console_accels(self):
+        console_pref = self.conf.get_bool(self.conf_dir +
+                                          "/console/enable-accels")
         if console_pref == None:
-            console_pref = 0
+            console_pref = False
         return console_pref
-    def set_console_keygrab(self, pref):
-        self.conf.set_int(self.conf_dir + "/console/keygrab", pref)
+    def set_console_accels(self, pref):
+        self.conf.set_bool(self.conf_dir + "/console/enable-accels", pref)
 
     def on_console_scaling_changed(self, callback):
         self.conf.notify_add(self.conf_dir + "/console/scaling", callback)
@@ -445,13 +474,6 @@ class vmmConfig:
     def set_console_scaling(self, pref):
         self.conf.set_int(self.conf_dir + "/console/scaling", pref)
 
-    # VNC console pointer grab notification
-    def show_console_grab_notify(self):
-        return self.conf.get_bool(self.conf_dir + "/console/grab-notify")
-    def set_console_grab_notify(self, state):
-        self.conf.set_bool(self.conf_dir + "/console/grab-notify", state)
-
-
     # Show VM details toolbar
     def get_details_show_toolbar(self):
         res = self.conf.get_bool(self.conf_dir + "/details/show-toolbar")
@@ -495,10 +517,10 @@ class vmmConfig:
 
         if urls.count(url) == 0 and len(url) > 0 and not url.isspace():
             # The url isn't already in the list, so add it
-            urls.insert(0,url)
+            urls.insert(0, url)
             length = self.get_url_list_length()
             if len(urls) > length:
-                del urls[len(urls) -1]
+                del urls[len(urls) - 1]
             self.conf.set_list(gconf_path, gconf.VALUE_STRING, urls)
 
     def add_media_url(self, url):
@@ -711,7 +733,9 @@ class vmmConfig:
         # one if the attributes match - which they will since UUID
         # is our unique key
 
-        secret = vmmSecret(self.get_secret_name(vm), password, { "uuid" : vm.get_uuid(), "hvuri": vm.get_connection().get_uri() })
+        secret = vmmSecret(self.get_secret_name(vm), password,
+                           {"uuid" : vm.get_uuid(),
+                            "hvuri": vm.get_connection().get_uri()})
         _id = self.keyring.add_secret(secret)
         if _id != None:
             self.conf.set_int(self.conf_dir + "/console/passwords/" + vm.get_uuid(), _id)
