@@ -20,14 +20,13 @@
 
 import gobject
 import gtk
-import gtk.glade
 
 import logging
 
 import virtManager.config as cfg
 import virtManager.uihelpers as uihelpers
 from virtManager.connection import vmmConnection
-from virtManager.error import vmmErrorDialog
+from virtManager.baseclass import vmmGObjectUI
 from virtManager.delete import vmmDeleteDialog
 from virtManager.graphwidgets import CellRendererSparkline
 from virtManager import util as util
@@ -69,20 +68,20 @@ class "GtkTreeView" style "treeview-style"
 gtk.rc_parse_string(rcstring)
 
 
-class vmmManager(gobject.GObject):
+class vmmManager(vmmGObjectUI):
     __gsignals__ = {
-        "action-show-connect":(gobject.SIGNAL_RUN_FIRST,
-                                  gobject.TYPE_NONE, []),
+        "action-show-connect": (gobject.SIGNAL_RUN_FIRST,
+                                gobject.TYPE_NONE, []),
         "action-show-console": (gobject.SIGNAL_RUN_FIRST,
-                                gobject.TYPE_NONE, (str,str)),
+                                gobject.TYPE_NONE, (str, str)),
         "action-show-terminal": (gobject.SIGNAL_RUN_FIRST,
-                                gobject.TYPE_NONE, (str,str)),
+                                gobject.TYPE_NONE, (str, str)),
         "action-refresh-console": (gobject.SIGNAL_RUN_FIRST,
-                                   gobject.TYPE_NONE, (str,str)),
+                                   gobject.TYPE_NONE, (str, str)),
         "action-refresh-terminal": (gobject.SIGNAL_RUN_FIRST,
-                                    gobject.TYPE_NONE, (str,str)),
+                                    gobject.TYPE_NONE, (str, str)),
         "action-show-details": (gobject.SIGNAL_RUN_FIRST,
-                                gobject.TYPE_NONE, (str,str)),
+                                gobject.TYPE_NONE, (str, str)),
         "action-show-about": (gobject.SIGNAL_RUN_FIRST,
                               gobject.TYPE_NONE, []),
         "action-show-host": (gobject.SIGNAL_RUN_FIRST,
@@ -110,24 +109,15 @@ class vmmManager(gobject.GObject):
         "action-show-help": (gobject.SIGNAL_RUN_FIRST,
                                gobject.TYPE_NONE, [str]),
         "action-migrate-domain": (gobject.SIGNAL_RUN_FIRST,
-                                  gobject.TYPE_NONE, (str,str)),
+                                  gobject.TYPE_NONE, (str, str)),
         "action-clone-domain": (gobject.SIGNAL_RUN_FIRST,
-                                gobject.TYPE_NONE, (str,str)),
+                                gobject.TYPE_NONE, (str, str)),
         "action-exit-app": (gobject.SIGNAL_RUN_FIRST,
-                            gobject.TYPE_NONE, []),}
+                            gobject.TYPE_NONE, []),
+    }
 
-    def __init__(self, config, engine):
-        self.__gobject_init__()
-        self.window = gtk.glade.XML((config.get_glade_dir() +
-                                     "/vmm-manager.glade"),
-                                     "vmm-manager", domain="virt-manager")
-        self.err = vmmErrorDialog(self.window.get_widget("vmm-manager"),
-                                  0, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE,
-                                  _("Unexpected Error"),
-                                  _("An unexpected error occurred"))
-        self.topwin = self.window.get_widget("vmm-manager")
-
-        self.config = config
+    def __init__(self, engine):
+        vmmGObjectUI.__init__(self, "vmm-manager.glade", "vmm-manager")
         self.engine = engine
 
         self.delete_dialog = None
@@ -139,6 +129,7 @@ class vmmManager(gobject.GObject):
 
         w, h = self.config.get_manager_window_size()
         self.topwin.set_default_size(w or 550, h or 550)
+        self.prev_position = None
 
         self.init_vmlist()
         self.init_stats()
@@ -194,10 +185,8 @@ class vmmManager(gobject.GObject):
         # Initialize stat polling columns based on global polling
         # preferences (we want signal handlers for this)
         for typ, init_val in \
-            [ (cfg.STATS_DISK,
-               self.config.get_stats_enable_disk_poll()),
-              (cfg.STATS_NETWORK,
-               self.config.get_stats_enable_net_poll())]:
+            [(cfg.STATS_DISK, self.config.get_stats_enable_disk_poll()),
+             (cfg.STATS_NETWORK, self.config.get_stats_enable_net_poll())]:
             self.enable_polling(None, None, init_val, typ)
 
         self.engine.connect("connection-added", self._add_connection)
@@ -217,24 +206,27 @@ class vmmManager(gobject.GObject):
     ##################
 
     def show(self):
-        if self.is_visible():
-            self.topwin.present()
-            return
+        vis = self.is_visible()
         self.topwin.present()
+        if vis:
+            return
 
+        if self.prev_position:
+            self.topwin.move(*self.prev_position)
+            self.prev_position = None
         self.engine.increment_window_counter()
 
-    def close(self, src=None, src2=None):
-        if self.is_visible():
-            win = self.window.get_widget("vmm-manager")
-            win.hide()
-            self.engine.decrement_window_counter()
-            return 1
+    def close(self, src_ignore=None, src2_ignore=None):
+        if not self.is_visible():
+            return
+
+        self.prev_position = self.topwin.get_position()
+        self.topwin.hide()
+        self.engine.decrement_window_counter()
+        return 1
 
     def is_visible(self):
-        if self.window.get_widget("vmm-manager").flags() & gtk.VISIBLE:
-            return 1
-        return 0
+        return bool(self.topwin.flags() & gtk.VISIBLE)
 
     def set_startup_error(self, msg):
         self.window.get_widget("vm-notebook").set_current_page(1)
@@ -270,7 +262,6 @@ class vmmManager(gobject.GObject):
         self.window.get_widget("vm-new").set_icon_name("vm_new")
         self.window.get_widget("vm-open").set_icon_name("icon_console")
         uihelpers.build_shutdown_button_menu(
-                                   self.config,
                                    self.window.get_widget("vm-shutdown"),
                                    self.poweroff_vm,
                                    self.reboot_vm,
@@ -518,29 +509,29 @@ class vmmManager(gobject.GObject):
 
         self.config.set_manager_window_size(event.width, event.height)
 
-    def exit_app(self, src=None, src2=None):
+    def exit_app(self, src_ignore=None, src2_ignore=None):
         self.emit("action-exit-app")
 
-    def new_connection(self, src=None):
+    def new_connection(self, src_ignore=None):
         self.emit("action-show-connect")
 
-    def new_vm(self, ignore=None):
+    def new_vm(self, src_ignore=None):
         self.emit("action-show-create", self.current_connection_uri())
 
-    def show_about(self, src):
+    def show_about(self, src_ignore):
         self.emit("action-show-about")
 
-    def show_help(self, src):
+    def show_help(self, src_ignore):
         self.emit("action-show-help", None)
 
-    def show_preferences(self, src):
+    def show_preferences(self, src_ignore):
         self.emit("action-show-preferences")
 
-    def show_host(self, src):
+    def show_host(self, src_ignore):
         uri = self.current_connection_uri(default_selection=True)
         self.emit("action-show-host", uri)
 
-    def open_vm_console(self,ignore,ignore2=None,ignore3=None):
+    def open_vm_console(self, ignore, ignore2=None, ignore3=None):
         if self.current_vmuuid():
             self.emit("action-show-console",
                       self.current_connection_uri(), self.current_vmuuid())
@@ -553,7 +544,7 @@ class vmmManager(gobject.GObject):
             self.emit("action-clone-domain", self.current_connection_uri(),
                       self.current_vmuuid())
 
-    def show_vm_details(self,ignore):
+    def show_vm_details(self, ignore):
         conn = self.current_connection()
         if conn is None:
             return
@@ -562,7 +553,7 @@ class vmmManager(gobject.GObject):
             self.emit("action-show-host", conn.get_uri())
         else:
             self.emit("action-show-console",
-                      conn.get_uri(), self.vm.get_uuid())
+                      conn.get_uri(), vm.get_uuid())
 
     def do_delete(self, ignore=None):
         conn = self.current_connection()
@@ -587,7 +578,7 @@ class vmmManager(gobject.GObject):
             return
 
         if not self.delete_dialog:
-            self.delete_dialog = vmmDeleteDialog(self.config, vm)
+            self.delete_dialog = vmmDeleteDialog(vm)
         else:
             self.delete_dialog.set_vm(vm)
 
@@ -669,7 +660,7 @@ class vmmManager(gobject.GObject):
         if conn.get_state() != vmmConnection.STATE_DISCONNECTED:
             conn.close()
 
-    def open_connection(self, ignore = None):
+    def open_connection(self, ignore=None):
         conn = self.current_connection()
         if conn.get_state() == vmmConnection.STATE_DISCONNECTED:
             conn.open()
@@ -677,21 +668,22 @@ class vmmManager(gobject.GObject):
 
     def _connect_error(self, conn, details):
         if conn.get_driver() == "xen" and not conn.is_remote():
-            self.err.show_err(_("Unable to open a connection to the Xen hypervisor/daemon.\n\n" +
-                              "Verify that:\n" +
-                              " - A Xen host kernel was booted\n" +
-                              " - The Xen service has been started\n"),
-                              details,
-                              title=_("Virtual Machine Manager Connection Failure"))
+            self.err.show_err(
+            _("Unable to open a connection to the Xen hypervisor/daemon.\n\n" +
+              "Verify that:\n" +
+              " - A Xen host kernel was booted\n" +
+              " - The Xen service has been started\n"),
+              details,
+              title=_("Virtual Machine Manager Connection Failure"))
         else:
-            self.err.show_err(_("Unable to open a connection to the libvirt "
-                                "management daemon.\n\n" +
-                                "Libvirt URI is: %s\n\n" % conn.get_uri() +
-                                "Verify that:\n" +
-                                " - The 'libvirtd' daemon has been started\n"),
-                              details,
-                              title=_("Virtual Machine Manager Connection "
-                                      "Failure"))
+            self.err.show_err(
+            _("Unable to open a connection to the libvirt "
+              "management daemon.\n\n" +
+              "Libvirt URI is: %s\n\n" % conn.get_uri() +
+              "Verify that:\n" +
+              " - The 'libvirtd' daemon has been started\n"),
+              details,
+              title=_("Virtual Machine Manager Connection Failure"))
 
 
     ####################################
@@ -701,7 +693,7 @@ class vmmManager(gobject.GObject):
     def vm_row_key(self, vm):
         return vm.get_uuid() + ":" + vm.get_connection().get_uri()
 
-    def vm_added(self, connection, uri, vmuuid):
+    def vm_added(self, connection, uri_ignore, vmuuid):
         vm = connection.get_vm(vmuuid)
         vm.connect("status-changed", self.vm_status_changed)
         vm.connect("resources-sampled", self.vm_resources_sampled)
@@ -711,7 +703,7 @@ class vmmManager(gobject.GObject):
 
         self._append_vm(model, vm, connection)
 
-    def vm_removed(self, connection, uri, vmuuid):
+    def vm_removed(self, connection, uri_ignore, vmuuid):
         vmlist = self.window.get_widget("vm-list")
         model = vmlist.get_model()
 
@@ -726,16 +718,17 @@ class vmmManager(gobject.GObject):
     def vm_started(self, connection, uri, vmuuid):
         vm = connection.get_vm(vmuuid)
         logging.debug("VM %s started" % vm.get_name())
-        if self.config.get_console_popup() == 2 and not vm.is_management_domain():
-            # user has requested consoles on all vms
-            gtype = vm.get_graphics_console()[0]
-            if gtype == "vnc":
-                self.emit("action-show-console", uri, vmuuid)
-            elif not connection.is_remote():
-                self.emit("action-show-terminal", uri, vmuuid)
-        else:
+        if (self.config.get_console_popup() != 2 or
+            vm.is_management_domain()):
             self.emit("action-refresh-console", uri, vmuuid)
 
+        # user has requested consoles on all vms
+        gtype = vm.get_graphics_console()[0]
+        if gtype in self.config.embeddable_graphics():
+            self.emit("action-show-console", uri, vmuuid)
+        elif not connection.is_remote():
+            self.emit("action-show-terminal", uri, vmuuid)
+
     def _build_conn_hint(self, conn):
         hint = conn.get_uri()
         if conn.state == conn.STATE_DISCONNECTED:
@@ -744,13 +737,13 @@ class vmmManager(gobject.GObject):
 
     def _build_conn_markup(self, conn, row):
         if conn.state == conn.STATE_DISCONNECTED:
-            markup = ("<span font_desc='9'>%s - "
-                      "Not Connected</span>" % row[ROW_NAME])
+            text = str(row[ROW_NAME]) + " - " + _("Not Connected")
         elif conn.state == conn.STATE_CONNECTING:
-            markup = ("<span font_desc='9'>%s - "
-                      "Connecting...</span>" % row[ROW_NAME])
+            text = str(row[ROW_NAME]) + " - " + _("Connecting...")
         else:
-            markup = ("<span font_desc='9'>%s</span>" % row[ROW_NAME])
+            text = str(row[ROW_NAME])
+
+        markup = "<span size='smaller'>%s</span>" % text
         return markup
 
     def _build_conn_color(self, conn):
@@ -763,14 +756,15 @@ class vmmManager(gobject.GObject):
         return color
 
     def _build_vm_markup(self, vm, row):
-        markup = ("<span font_desc='10'>%s</span>\n"
-                  "<span font_desc='8'>%s</span>" %
-                  (row[ROW_NAME], row[ROW_STATUS]))
-        return markup
+        ignore = vm
+        domtext     = ("<span size='smaller' weight='bold'>%s</span>" %
+                       row[ROW_NAME])
+        statetext   = "<span size='smaller'>%s</span>" % row[ROW_STATUS]
+        return domtext + "\n" + statetext
 
     def _append_vm(self, model, vm, conn):
         row_key = self.vm_row_key(vm)
-        if self.rows.has_key(row_key):
+        if row_key in self.rows:
             return
 
         parent = self.rows[conn.get_uri()].iter
@@ -801,7 +795,7 @@ class vmmManager(gobject.GObject):
         row.insert(ROW_HANDLE, conn)
         row.insert(ROW_NAME, conn.get_pretty_desc_inactive(False))
         row.insert(ROW_MARKUP, self._build_conn_markup(conn, row))
-        row.insert(ROW_STATUS, ("<span font_desc='9'>%s</span>" %
+        row.insert(ROW_STATUS, ("<span size='smaller'>%s</span>" %
                                 conn.get_state_text()))
         row.insert(ROW_STATUS_ICON, None)
         row.insert(ROW_KEY, conn.get_uri())
@@ -818,11 +812,11 @@ class vmmManager(gobject.GObject):
         self.rows[conn.get_uri()] = model[path]
         return _iter
 
-    def _add_connection(self, engine, conn):
+    def _add_connection(self, engine_ignore, conn):
         # Make sure error page isn't showing
         self.window.get_widget("vm-notebook").set_current_page(0)
 
-        if self.rows.has_key(conn.get_uri()):
+        if conn.get_uri() in self.rows:
             return
 
         conn.connect("vm-added", self.vm_added)
@@ -837,7 +831,25 @@ class vmmManager(gobject.GObject):
         row = self._append_connection(vmlist.get_model(), conn)
         vmlist.get_selection().select_iter(row)
 
-    def _remove_connection(self, engine, conn):
+        # Try to make sure that 2 row descriptions don't collide
+        connrows = []
+        descs = []
+        for row in self.rows.values():
+            if row[ROW_IS_CONN]:
+                connrows.append(row)
+        for row in connrows:
+            descs.append(row[ROW_NAME])
+
+        for row in connrows:
+            conn = row[ROW_HANDLE]
+            name = row[ROW_NAME]
+            if descs.count(name) <= 1:
+                continue
+
+            newname = conn.get_pretty_desc_inactive(False, True)
+            self.conn_refresh_resources(conn, newname)
+
+    def _remove_connection(self, engine_ignore, conn):
         model = self.window.get_widget("vm-list").get_model()
         parent = self.rows[conn.get_uri()].iter
         if parent is not None:
@@ -854,7 +866,7 @@ class vmmManager(gobject.GObject):
     # State/UI updating methods #
     #############################
 
-    def vm_status_changed(self, vm, status, ignore):
+    def vm_status_changed(self, vm, status_ignore, oldstatus_ignore):
         parent = self.rows[vm.get_connection().get_uri()].iter
 
         vmlist = self.window.get_widget("vm-list")
@@ -877,7 +889,7 @@ class vmmManager(gobject.GObject):
         vmlist = self.window.get_widget("vm-list")
         model = vmlist.get_model()
 
-        if not self.rows.has_key(self.vm_row_key(vm)):
+        if self.vm_row_key(vm) not in self.rows:
             return
 
         row = self.rows[self.vm_row_key(vm)]
@@ -891,13 +903,16 @@ class vmmManager(gobject.GObject):
         self.conn_refresh_resources(conn)
         self.vm_selected()
 
-    def conn_refresh_resources(self, conn):
+    def conn_refresh_resources(self, conn, newname=None):
         vmlist = self.window.get_widget("vm-list")
         model = vmlist.get_model()
         row = self.rows[conn.get_uri()]
 
+        if newname:
+            row[ROW_NAME] = newname
         row[ROW_MARKUP] = self._build_conn_markup(conn, row)
-        row[ROW_STATUS] = "<span font_desc='9'>%s</span>" % conn.get_state_text()
+        row[ROW_STATUS] = ("<span size='smaller'>%s</span>" %
+                           conn.get_state_text())
         row[ROW_IS_CONN_CONNECTED] = conn.state != conn.STATE_DISCONNECTED
         row[ROW_HINT] = self._build_conn_hint(conn)
         row[ROW_COLOR] = self._build_conn_color(conn)
@@ -955,7 +970,7 @@ class vmmManager(gobject.GObject):
         self.window.get_widget("menu_host_details").set_sensitive(host_details)
         self.window.get_widget("menu_edit_delete").set_sensitive(delete)
 
-    def popup_vm_menu_key(self, widget, event):
+    def popup_vm_menu_key(self, widget_ignore, event):
         if gtk.gdk.keyval_name(event.keyval) != "Menu":
             return False
 
@@ -1044,8 +1059,11 @@ class vmmManager(gobject.GObject):
             widgn = "menu_view_stats_network"
         widget = self.window.get_widget(widgn)
 
+        disabled_text = _(" (disabled)")
         tool_text = ""
-        if conf_entry and (conf_entry == True or \
+        current_text = widget.get_label().strip(disabled_text)
+
+        if conf_entry and (conf_entry == True or
                            conf_entry.get_value().get_bool()):
             widget.set_sensitive(True)
         else:
@@ -1053,8 +1071,10 @@ class vmmManager(gobject.GObject):
                 widget.set_active(False)
             widget.set_sensitive(False)
             tool_text = _("Disabled in preferences dialog.")
+            current_text = current_text + disabled_text
 
         util.tooltip_wrapper(widget, tool_text)
+        widget.set_label(current_text)
 
     def toggle_network_traffic_visible_widget(self, *ignore):
         vmlist = self.window.get_widget("vm-list")
@@ -1080,22 +1100,22 @@ class vmmManager(gobject.GObject):
         }
         set_stats[stats_id](visible)
 
-    def cpu_usage_img(self,  column, cell, model, _iter, data):
+    def cpu_usage_img(self, column_ignore, cell, model, _iter, data):
         if model.get_value(_iter, ROW_HANDLE) is None:
             return
         data = model.get_value(_iter, ROW_HANDLE).cpu_time_vector_limit(40)
         cell.set_property('data_array', data)
 
-    def disk_io_img(self,  column, cell, model, _iter, data):
+    def disk_io_img(self, column_ignore, cell, model, _iter, data):
         if model.get_value(_iter, ROW_HANDLE) is None:
             return
         data = model.get_value(_iter, ROW_HANDLE).disk_io_vector_limit(40)
         cell.set_property('data_array', data)
 
-    def network_traffic_img(self,  column, cell, model, _iter, data):
+    def network_traffic_img(self, column_ignore, cell, model, _iter, data):
         if model.get_value(_iter, ROW_HANDLE) is None:
             return
         data = model.get_value(_iter, ROW_HANDLE).network_traffic_vector_limit(40)
         cell.set_property('data_array', data)
 
-gobject.type_register(vmmManager)
+vmmGObjectUI.type_register(vmmManager)
