@@ -27,25 +27,35 @@ import libxml2
 import logging
 import os.path
 
+from virtManager.config import running_config
 import virtManager
 import virtinst
 
-DEFAULT_POOL_NAME = "default"
-DEFAULT_POOL_PATH = "/var/lib/libvirt/images"
+# FIXME: selinux policy also has a ~/VirtualMachines/isos dir
+def get_default_pool_path(conn):
+    if conn.is_session_uri():
+        return os.path.expanduser("~/VirtualMachines")
+    return "/var/lib/libvirt/images"
 
-def build_default_pool(conn):
+def get_default_pool_name(conn):
+    ignore = conn
+    return "default"
+
+def build_default_pool(vmmconn):
     """
     Helper to build the 'default' storage pool
     """
     # FIXME: This should use config.get_default_image_path ?
-
+    conn = vmmconn.vmm
     if not virtinst.util.is_storage_capable(conn):
         # VirtualDisk will raise an error for us
         return
 
+    path = get_default_pool_path(vmmconn)
+    name = get_default_pool_name(vmmconn)
     pool = None
     try:
-        pool = conn.storagePoolLookupByName(DEFAULT_POOL_NAME)
+        pool = conn.storagePoolLookupByName(name)
     except libvirt.libvirtError:
         pass
 
@@ -54,46 +64,47 @@ def build_default_pool(conn):
 
     try:
         logging.debug("Attempting to build default pool with target '%s'" %
-                      DEFAULT_POOL_PATH)
+                      path)
         defpool = virtinst.Storage.DirectoryPool(conn=conn,
-                                                 name=DEFAULT_POOL_NAME,
-                                                 target_path=DEFAULT_POOL_PATH)
+                                                 name=name,
+                                                 target_path=path)
         newpool = defpool.install(build=True, create=True)
         newpool.setAutostart(True)
     except Exception, e:
         raise RuntimeError(_("Couldn't create default storage pool '%s': %s") %
-                             (DEFAULT_POOL_PATH, str(e)))
+                             (path, str(e)))
 
-def get_ideal_path_info(conn, config, name):
-    path = get_default_dir(conn, config)
+def get_ideal_path_info(conn, name):
+    path = get_default_dir(conn)
     suffix = ".img"
     return (path, name, suffix)
 
-def get_ideal_path(conn, config, name):
-    target, name, suffix = get_ideal_path_info(conn, config, name)
+def get_ideal_path(conn, name):
+    target, name, suffix = get_ideal_path_info(conn, name)
     return os.path.join(target, name) + suffix
 
 def get_default_pool(conn):
     pool = None
+    default_name = get_default_pool_name(conn)
     for uuid in conn.list_pool_uuids():
         p = conn.get_pool(uuid)
-        if p.get_name() == DEFAULT_POOL_NAME:
+        if p.get_name() == default_name:
             pool = p
 
     return pool
 
-def get_default_dir(conn, config):
+def get_default_dir(conn):
     pool = get_default_pool(conn)
 
     if pool:
         return pool.get_target_path()
     else:
-        return config.get_default_image_dir(conn)
+        return running_config.get_default_image_dir(conn)
 
-def get_default_path(conn, config, name):
+def get_default_path(conn, name):
     pool = get_default_pool(conn)
 
-    default_dir = get_default_dir(conn, config)
+    default_dir = get_default_dir(conn)
 
     if not pool:
         # Use old generating method
@@ -111,7 +122,7 @@ def get_default_path(conn, config, name):
 
         path = f
     else:
-        target, ignore, suffix = get_ideal_path_info(conn, config, name)
+        target, ignore, suffix = get_ideal_path_info(conn, name)
 
         path = virtinst.Storage.StorageVolume.find_free_name(name,
                         pool_object=pool.pool, suffix=suffix)
@@ -153,7 +164,7 @@ def xml_parse_wrapper(xml, parse_func, *args, **kwargs):
     return ret
 
 
-def browse_local(parent, dialog_name, config, conn, start_folder=None,
+def browse_local(parent, dialog_name, conn, start_folder=None,
                  _type=None, dialog_type=gtk.FILE_CHOOSER_ACTION_OPEN,
                  confirm_func=None, browse_reason=None):
     """
@@ -161,7 +172,6 @@ def browse_local(parent, dialog_name, config, conn, start_folder=None,
 
     @param parent: Parent window for the filechooser
     @param dialog_name: String to use in the title bar of the filechooser.
-    @param config: vmmConfig used by calling class
     @param conn: vmmConnection used by calling class
     @param start_folder: Folder the filechooser is viewing at startup
     @param _type: File extension to filter by (e.g. "iso", "png")
@@ -209,7 +219,8 @@ def browse_local(parent, dialog_name, config, conn, start_folder=None,
 
     # Set initial dialog folder
     if browse_reason:
-        start_folder = config.get_default_directory(conn, browse_reason)
+        start_folder = running_config.get_default_directory(conn,
+                                                            browse_reason)
 
     if start_folder != None:
         if os.access(start_folder, os.R_OK):
@@ -228,19 +239,20 @@ def browse_local(parent, dialog_name, config, conn, start_folder=None,
 
     # Store the chosen directory in gconf if necessary
     if ret and browse_reason and not ret.startswith("/dev"):
-        config.set_default_directory(os.path.dirname(ret), browse_reason)
-
+        running_config.set_default_directory(os.path.dirname(ret),
+                                             browse_reason)
     return ret
 
-def dup_lib_conn(config, libconn):
-    return _dup_all_conn(config, None, libconn=libconn,
-                         return_conn_class=False)
+def dup_lib_conn(libconn):
+    conn = _dup_all_conn(None, libconn)
+    if isinstance(conn, virtManager.connection.vmmConnection):
+        return conn.vmm
+    return conn
 
-def dup_conn(config, conn, return_conn_class=False):
-    return _dup_all_conn(config, conn, None,
-                         return_conn_class=return_conn_class)
+def dup_conn(conn):
+    return _dup_all_conn(conn, None)
 
-def _dup_all_conn(config, conn, libconn, return_conn_class):
+def _dup_all_conn(conn, libconn):
 
     is_readonly = False
 
@@ -257,22 +269,18 @@ def _dup_all_conn(config, conn, libconn, return_conn_class):
     if is_test:
         # Skip duplicating a test conn, since it doesn't maintain state
         # between instances
-        return return_conn_class and conn or vmm
+        return conn or vmm
 
     if virtinst.support.support_threading():
         # Libvirt 0.6.0 implemented client side request threading: this
         # removes the need to actually duplicate the connection.
-        return return_conn_class and conn or vmm
+        return conn or vmm
 
     logging.debug("Duplicating connection for async operation.")
-    newconn = virtManager.connection.vmmConnection(config, uri, is_readonly)
-    newconn.open()
-    newconn.connectThreadEvent.wait()
+    newconn = virtManager.connection.vmmConnection(uri, readOnly=is_readonly)
+    newconn.open(sync=True)
 
-    if return_conn_class:
-        return newconn
-    else:
-        return newconn.vmm
+    return newconn
 
 def pretty_hv(gtype, domtype):
     """
@@ -354,7 +362,8 @@ def safe_timeout_add(timeout, func, *args):
     return gobject.timeout_add(timeout, _safe_wrapper, func, *args)
 
 def uuidstr(rawuuid):
-    hx = ['0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f']
+    hx = ['0', '1', '2', '3', '4', '5', '6', '7',
+          '8', '9', 'a', 'b', 'c', 'd', 'e', 'f']
     uuid = []
     for i in range(16):
         uuid.append(hx[((ord(rawuuid[i]) >> 4) & 0xf)])
@@ -364,7 +373,7 @@ def uuidstr(rawuuid):
     return "".join(uuid)
 
 def bind_escape_key_close(vmmobj):
-    def close_on_escape(src, event):
+    def close_on_escape(src_ignore, event):
         if gtk.gdk.keyval_name(event.keyval) == "Escape":
             vmmobj.close()
 
@@ -394,4 +403,3 @@ def iface_in_use_by(conn, name):
             use_str += iface.get_name()
 
     return use_str
-
